/*
 * Copyright (c) 2018 Arnaud Ysmal.  All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

package themes

import (
	"fmt"
	"io"
	"jsonresume/model"
	"path"
	"strings"
	"text/template"
)

type Theme struct {
	Name      string
	Directory string
	Functions template.FuncMap
	Template  string
}

var Themes map[string]*Theme

func init() {
	Themes = make(map[string]*Theme)
}

func (t *Theme) Render(r *model.Resume, w io.Writer) error {
	tmpl := template.Must(template.New(t.Template).Funcs(t.Functions).ParseFiles(path.Join(t.Directory, t.Template)))
	return tmpl.Execute(w, r)
}

func (t *Theme) Register() error {
	if _, exists := Themes[t.Name]; exists {
		return fmt.Errorf("Package %s already registered", t.Name)
	}
	Themes[t.Name] = t
	return nil
}

// Common functions
func IconClass(network string) string {
	network = strings.ToLower(network)
	switch network {
	case "google-plus":
	case "googleplus":
		return "fa fa-google-plus"
	case "flickr":
	case "flicker":
		return "fa fa-flickr"
	case "dribbble":
	case "dribble":
		return "fa fa-dribbble"
	case "codepen":
		return "fa fa-codepen"
	case "soundcloud":
		return "fa fa-soundcloud"
	case "reddit":
		return "fa fa-reddit"
	case "tumblr":
	case "tumbler":
		return "fa fa-tumblr"
	case "stack-overflow":
	case "stackoverflow":
		return "fa fa-stack-overflow"
	case "blog":
	case "rss":
		return "fa fa-rss"
	case "gitlab":
		return "fa fa-gitlab"
	case "keybase":
		return "fa fa-key"
	default:
		return "fa fa-" + network
	}

	return "fa fa-" + network
}

func FormatDateWork(date model.ResumeDate) string {
	if date.IsZero() {
		return "Present"
	}
	return fmt.Sprintf("%s %d", date.Month().String(), date.Year())
}

func FormatDateEdu(date model.ResumeDate) string {
	if date.IsZero() {
		return "Present"
	}
	return fmt.Sprintf("%d", date.Year())
}

func FormatDatePub(date model.ResumeDate) string {
	if date.IsZero() {
		return "Present"
	}
	return fmt.Sprintf("%d %s %d", date.Day(), date.Month().String(), date.Year())
}
